@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 #
 # Download files from file sharing websites
-# Copyright (c) 2010-2016 Plowshare team
+# Copyright (c) 2010-2014 Plowshare team
 #
 # This file is part of Plowshare.
 #
@@ -22,12 +22,11 @@ declare -r VERSION='GIT-snapshot'
 
 declare -r EARLY_OPTIONS="
 HELP,h,help,,Show help info and exit
-HELPFUL,H,longhelp,,Exhaustive help info (with modules command-line options)
+HELPFULL,H,longhelp,,Exhaustive help info (with modules command-line options)
 GETVERSION,,version,,Output plowdown version information and exit
 ALLMODULES,,modules,,Output available modules (one per line) and exit. Useful for wrappers.
 EXT_PLOWSHARERC,,plowsharerc,f=FILE,Force using an alternate configuration file (overrides default search path)
-NO_PLOWSHARERC,,no-plowsharerc,,Do not use any plowshare.conf configuration file
-NO_COLOR,,no-color,,Disables log notice & log error output coloring"
+NO_PLOWSHARERC,,no-plowsharerc,,Do not use any plowshare.conf configuration file"
 
 declare -r MAIN_OPTIONS="
 VERBOSE,v,verbose,c|0|1|2|3|4=LEVEL,Verbosity level: 0=none, 1=err, 2=notice (default), 3=dbg, 4=report
@@ -38,7 +37,6 @@ OUTPUT_DIR,o,output-directory,D=DIR,Directory where files will be saved
 TEMP_DIR,,temp-directory,D=DIR,Directory for temporary files (final link download, cookies, images)
 TEMP_RENAME,,temp-rename,,Append .part suffix to filename while file is being downloaded
 MAX_LIMIT_RATE,,max-rate,r=SPEED,Limit maximum speed to bytes/sec (accept usual suffixes)
-MIN_LIMIT_RATE,,min-rate,r=SPEED,Limit minimum speed to bytes/sec (during 30 seconds)
 MIN_LIMIT_SPACE,,min-space,R=LIMIT,Set the minimum amount of disk space to exit.
 INTERFACE,i,interface,s=IFACE,Force IFACE network interface
 TIMEOUT,t,timeout,n=SECS,Timeout after SECS seconds of waits
@@ -54,9 +52,8 @@ CAPTCHA_DEATHBY,,deathbycaptcha,a=USER:PASSWD,DeathByCaptcha account
 PRE_COMMAND,,run-before,F=PROGRAM,Call external program/script before new link processing
 POST_COMMAND,,run-after,F=PROGRAM,Call external program/script after link being successfully processed
 SKIP_FINAL,,skip-final,,Don't process final link (returned by module), just skip it (for each link)
-PRINTF_FORMAT,,printf,s=FORMAT,Print results in a given format (for each successful download). Default is \"%F%n\".
+PRINTF_FORMAT,,printf,s=FORMAT,Print results in a given format (for each successful download). Default string is: \"%F%n\".
 NO_MODULE_FALLBACK,,fallback,,If no module is found for link, simply download it (HTTP GET)
-EXT_CURLRC,,curlrc,f=FILE,Force using an alternate curl configuration file (overrides ~/.curlrc)
 NO_CURLRC,,no-curlrc,,Do not use curlrc config file"
 
 
@@ -143,19 +140,10 @@ mark_queue() {
         if [ 'file' = "$1" ]; then
             if test -w "$FILE"; then
                 local -r D=$'\001' # sed separator
-                local SRET=0
-                if [ -z "$FILENAME" ]; then
-                    sed -i -e "s$D^[[:space:]]*\(${URL//\\/\\\\/}[[:space:]]*\)\$$D$STATUS \1$D" "$FILE" || SRET=$?
-                else
-                    # Don't write filename if it is already present
-                    sed -i -e "\\$D^[[:space:]]*#[[:space:]]*${FILENAME:2}\r\?\$$D{N}" \
-                        -e "s$D^\([[:space:]]*#[[:space:]]*${FILENAME:2}\r\?\n\)\?[[:space:]]*\(${URL//\\/\\\\/}[[:space:]]*\r\?\)\$$D${FILENAME//&/\\&}\n$STATUS \2$D" "$FILE" || SRET=$?
-                fi
-                if [ $SRET -eq 0 ]; then
-                    log_notice "link marked in file \`$FILE' ($STATUS)"
-                else
+                test "$FILENAME" && FILENAME="${FILENAME//&/\\&}\n"
+                sed -i -e "s$D^[[:space:]]*\(${URL//\\/\\\\/}[[:space:]]*\)\$$D$FILENAME$STATUS \1$D" "$FILE" &&
+                    log_notice "link marked in file \`$FILE' ($STATUS)" ||
                     log_error "failed marking link in file \`$FILE' ($STATUS)"
-                fi
             else
                 log_error "Can't mark link, no write permission ($FILE)"
             fi
@@ -307,8 +295,6 @@ download() {
             DRETVAL=0
             $FUNCTION "$COOKIE_FILE" "$URL_ENCODED" >"$DRESULT" || DRETVAL=$?
 
-            # $ERR_LINK_TEMP_UNAVAILABLE and $ERR_EXPIRED_SESSION
-            # do not count as a retry
             if [ $DRETVAL -eq $ERR_LINK_TEMP_UNAVAILABLE ]; then
                 read AWAIT <"$DRESULT"
                 if [ -z "$AWAIT" ]; then
@@ -318,9 +304,6 @@ download() {
                 fi
                 wait ${AWAIT:-60} || { DRETVAL=$?; break; }
                 continue
-            elif [ $DRETVAL -eq $ERR_EXPIRED_SESSION ]; then
-                log_notice 'expired session: delete cache entry'
-                continue
             elif [[ $MAX_RETRIES -eq 0 ]]; then
                 break
             elif [ $DRETVAL -ne $ERR_NETWORK -a \
@@ -431,7 +414,7 @@ download() {
         if [[ $BASE_URL =~ :([[:digit:]]{2,5})$ ]]; then
             local -i PORT=${BASH_REMATCH[1]}
             if (( PORT != 80 && PORT != 443 )); then
-                log_notice "WARNING: Final URL requires an outgoing TCP connection to port $PORT, hope you're not behind a proxy/firewall."
+                log_notice "Warning: Final URL requires an outgoing TCP connection to port $PORT, hope you're not behind a proxy/firewall"
             fi
         fi
 
@@ -508,11 +491,9 @@ download() {
                 if [ -f "$FILENAME_OUT" ]; then
                     # Can we overwrite destination file?
                     if [ ! -w "$FILENAME_OUT" ]; then
-                        if module_config_resume "$MODULE"; then
-                            log_error "ERROR: No write permission, cannot resume final file ($FILENAME_OUT)"
-                        else
-                            log_error "ERROR: No write permission, cannot overwrite final file ($FILENAME_OUT)"
-                        fi
+                        module_config_resume "$MODULE" && \
+                            log_error "error: no write permission, cannot resume final file ($FILENAME_OUT)" || \
+                            log_error "error: no write permission, cannot overwrite final file ($FILENAME_OUT)"
                         return $ERR_SYSTEM
                     fi
 
@@ -525,20 +506,18 @@ download() {
                 if [ -f "$FILENAME_OUT" ]; then
                     # Can we overwrite destination file?
                     if [ ! -w "$FILENAME_OUT" ]; then
-                        log_error "ERROR: No write permission, cannot overwrite final file ($FILENAME_OUT)"
+                        log_error "error: no write permission, cannot overwrite final file ($FILENAME_OUT)"
                         return $ERR_SYSTEM
                     fi
-                    log_notice 'WARNING: The filename file already exists, overwrite it. Use `plowdown --no-overwrite'\'' to disable.'
+                    log_notice "warning: final file will be overwritten ($FILENAME_OUT)"
                 fi
 
                 if [ -f "$FILENAME_TMP" ]; then
                     # Can we overwrite temporary file?
                     if [ ! -w "$FILENAME_TMP" ]; then
-                        if module_config_resume "$MODULE"; then
-                            log_error "ERROR: No write permission, cannot resume tmp/part file ($FILENAME_TMP)"
-                        else
-                            log_error "ERROR: No write permission, cannot overwrite tmp/part file ($FILENAME_TMP)"
-                        fi
+                        module_config_resume "$MODULE" && \
+                            log_error "error: no write permission, cannot resume tmp/part file ($FILENAME_TMP)" || \
+                            log_error "error: no write permission, cannot overwrite tmp/part file ($FILENAME_TMP)"
                         return $ERR_SYSTEM
                     fi
 
@@ -556,14 +535,8 @@ download() {
             if ANAME=$(module_config_need_extra "$MODULE"); then
                 local OPTION
                 for OPTION in "${!ANAME}"; do
-                    if [[ $OPTION = '-J' || $OPTION = '--remote-header-name' ]]; then
-                        log_debug "ignoring extra curl option: '$OPTION'"
-                    elif [[ $OPTION = '-O' || $OPTION = '--remote-name' ]]; then
-                        log_debug "ignoring extra curl option: '$OPTION'"
-                    else
-                        log_debug "adding extra curl option: '$OPTION'"
-                        CURL_ARGS+=("$OPTION")
-                    fi
+                    log_debug "adding extra curl options: '$OPTION'"
+                    CURL_ARGS+=("$OPTION")
                 done
             fi
 
@@ -575,7 +548,7 @@ download() {
             fi
 
             DRETVAL=0
-            umask 0066 && curl_with_log "${CURL_ARGS[@]}" --fail --globoff \
+            curl_with_log "${CURL_ARGS[@]}" --fail --globoff \
                 -w '%{http_code}\t%{size_download}' \
                 -o "$FILENAME_TMP" "$FILE_URL" >"$DRESULT" || DRETVAL=$?
             IFS=$'\t' read -r STATUS FILE_SIZE < "$DRESULT"
@@ -627,12 +600,12 @@ download() {
                 return $ERR_NETWORK
             fi
 
-            chmod 644 "$FILENAME_TMP" 2>/dev/null || log_error "chmod failed: $FILENAME_TMP"
+            chmod 644 "$FILENAME_TMP" || log_error "chmod failed: $FILENAME_TMP"
 
             if [ "$FILENAME_TMP" != "$FILENAME_OUT" ]; then
                 test "$TEMP_RENAME" || \
                     log_notice "Moving file to output directory: ${OUT_DIR:-.}"
-                mv -f "$FILENAME_TMP" "$FILENAME_OUT" 2>/dev/null || log_error "mv failed: $FILENAME_TMP"
+                mv -f "$FILENAME_TMP" "$FILENAME_OUT"
             fi
 
             mark_queue "$TYPE" "$MARK_DOWN" "$ITEM" "$URL_RAW" OK "$FILENAME_OUT"
@@ -669,13 +642,11 @@ download() {
 # %c: final cookie filename (with output directory)
 # %C: %c or empty string if module does not require it
 # %d: download (final) url
-# %D: download (final) url (JSON string)
 # %f: destination (local) filename
 # %F: destination (local) filename (with output directory)
 # %m: module name
 # %s: destination (local) file size (in bytes)
 # %u: download (source) url
-# %U: download url (JSON string)
 # and also:
 # %n: newline
 # %t: tabulation
@@ -687,7 +658,7 @@ download() {
 pretty_check() {
     # This must be non greedy!
     local S TOKEN
-    S=${1//%[cdDfmsuUCFnt%]}
+    S=${1//%[cdfmsuCFnt%]}
     TOKEN=$(parse_quiet . '\(%.\)' <<< "$S")
     if [ -n "$TOKEN" ]; then
         log_error "Bad format string: unknown sequence << $TOKEN >>"
@@ -700,12 +671,12 @@ pretty_check() {
 # $3: format string
 # Note: Don't chmod cookie file (keep strict permissions)
 pretty_print() {
+    local -r N=$(printf %04d $1)
     local -ar A=("${!2}")
     local -r CR=$'\n'
     local FMT=$3
-    local N COOKIE_FILE
+    local COOKIE_FILE
 
-    printf -v N %04d $(($1))
     test "${FMT#*%%}" != "$FMT" && FMT=$(replace_all '%%' "%raw" <<< "$FMT")
 
     # FIXME: ${A[2]} could contain %? patterns
@@ -743,7 +714,7 @@ pretty_print() {
 
     handle_tokens "$FMT" '%raw,%' '%t,	' "%n,$CR" \
         "%m,${A[0]}" "%f,${A[1]}" "%u,${A[4]}" "%d,${A[5]}" \
-        "%s,${A[6]}" "%U,$(json_escape "${A[4]}")" "%D,$(json_escape "${A[5]}")"
+        "%s,${A[6]}"
 }
 
 #
@@ -770,18 +741,15 @@ TMPDIR=${TMPDIR:-/tmp}
 set -e # enable exit checking
 
 source "$LIBDIR/core.sh"
-
-declare -a MODULES=()
-eval "$(get_all_modules_list download)" || exit
-for MODULE in "${!MODULES_PATH[@]}"; do
-    source "${MODULES_PATH[$MODULE]}"
-    MODULES+=("$MODULE")
+mapfile -t MODULES < <(get_all_modules_list "$LIBDIR" 'download') || exit
+for MODULE in "${MODULES[@]}"; do
+    source "$LIBDIR/modules/$MODULE.sh"
 done
 
 # Process command-line (plowdown early options)
 eval "$(process_core_options 'plowdown' "$EARLY_OPTIONS" "$@")" || exit
 
-test "$HELPFUL" && { usage 1; exit 0; }
+test "$HELPFULL" && { usage 1; exit 0; }
 test "$HELP" && { usage; exit 0; }
 test "$GETVERSION" && { echo "$VERSION"; exit 0; }
 
@@ -792,14 +760,7 @@ fi
 
 # Get configuration file options. Command-line is partially parsed.
 test -z "$NO_PLOWSHARERC" && \
-    process_configfile_options '[Pp]lowdown' "${MAIN_OPTIONS}
-NO_COLOR,,no-color,,x" "$EXT_PLOWSHARERC"
-
-if [ -n "$NO_COLOR" ]; then
-    unset COLOR
-else
-    declare -r COLOR=yes
-fi
+    process_configfile_options '[Pp]lowdown' "$MAIN_OPTIONS" "$EXT_PLOWSHARERC"
 
 declare -a COMMAND_LINE_MODULE_OPTS COMMAND_LINE_ARGS RETVALS
 COMMAND_LINE_ARGS=("${UNUSED_ARGS[@]}")
@@ -815,18 +776,6 @@ elif [ -z "$VERBOSE" ]; then
     declare -r VERBOSE=2
 fi
 
-if [ "${#MODULES}" -le 0 ]; then
-    log_error \
-"-------------------------------------------------------------------------------
-Your plowshare installation has currently no module.
-($PLOWSHARE_CONFDIR/modules.d/ is empty)
-
-In order to use plowdown you must install some modules. Here is a quick start:
-$ plowmod --install
--------------------------------------------------------------------------------"
-    exit $ERR_NOMODULE
-fi
-
 if [ $# -lt 1 ]; then
     log_error 'plowdown: no URL specified!'
     log_error "plowdown: try \`plowdown --help' for more information."
@@ -844,13 +793,6 @@ if [ -n "$EXT_PLOWSHARERC" ]; then
     fi
 fi
 
-if [ -n "$MAX_LIMIT_RATE" -a -n "$MIN_LIMIT_RATE" ]; then
-  if (( MAX_LIMIT_RATE < MIN_LIMIT_RATE )); then
-      log_error "--min-rate ($MIN_LIMIT_RATE) is greater than --max-rate ($MAX_LIMIT_RATE)"
-      exit $ERR_BAD_COMMAND_LINE
-  fi
-fi
-
 if [ -n "$TEMP_DIR" ]; then
     TMPDIR=${TEMP_DIR%/}
     log_notice "Temporary directory: $TMPDIR"
@@ -859,7 +801,7 @@ fi
 if [ -n "$OUTPUT_DIR" ]; then
     log_notice "Output directory: ${OUTPUT_DIR%/}"
 elif [ ! -w "$PWD" ]; then
-    log_notice 'WARNING: Current directory is not writable, you may experience troubles.'
+    log_notice 'Warning: Current directory is not writable!'
 fi
 
 if [ -n "$MIN_LIMIT_SPACE" ]; then
@@ -868,8 +810,6 @@ fi
 
 if [ -n "$PRINTF_FORMAT" ]; then
     pretty_check "$PRINTF_FORMAT" || exit
-elif [ -n "$SKIP_FINAL" -a -z "$POST_COMMAND" ]; then
-    log_notice 'plowdown: using --skip-final without --printf is probably not what you want'
 fi
 
 # Print chosen options
@@ -889,13 +829,7 @@ else
     [ -n "$CAPTCHA_DEATHBY" ] && log_debug 'plowdown: --deathbycaptcha selected'
 fi
 
-if [ -n "$EXT_CURLRC" ]; then
-    if [ -n "$NO_CURLRC" ]; then
-        log_notice 'plowdown: --no-curlrc selected and prevails over --curlrc'
-    else
-        log_notice 'plowdown: using alternate curl configuration file'
-    fi
-elif [ -z "$NO_CURLRC" -a -f "$HOME/.curlrc" ]; then
+if [ -z "$NO_CURLRC" -a -f "$HOME/.curlrc" ]; then
     log_debug 'using local ~/.curlrc'
 fi
 
@@ -915,16 +849,16 @@ if [ ${#COMMAND_LINE_ARGS[@]} -eq 0 ]; then
     exit $ERR_BAD_COMMAND_LINE
 fi
 
-core_init 'plowdown'
+set_exit_trap
+
+# Save umask
+declare -r UMASK=$(umask)
+test "$UMASK" && umask 0066
 
 # Remember last host because hosters may require waiting between
-# successive downloads.
+# sucessive downloads.
 PREVIOUS_HOST=none
 
-# Only used when CACHE policy is session (default).
-# Use an associative array to not have duplicated modules.
-declare -A CACHED_MODULES
-
 # Count downloads (1-based index)
 declare -i INDEX=1
 
@@ -949,18 +883,9 @@ for ITEM in "${COMMAND_LINE_ARGS[@]}"; do
             log_notice "This seems to be a redirection url. Trying with: '$URL'"
         fi
 
-        # Sanity check
-        if [[ ${URL%/} =~ ^[Hh][Tt][Tt][Pp][Ss]?://(www\.)?[[:alnum:]]+\.[[:alpha:]]{2,3}$ ]]; then
-            log_notice 'You seem to have entered a basename link without any path/query. Please check if your link is valid.'
-            URL="${URL%/}/"
-            # Force error even if $MODULE detected
-            MRETVAL=$ERR_NOMODULE
-        fi
-
         MODULE=$(get_module "$URL" MODULES[@]) || true
 
         if [ -z "$MODULE" ]; then
-            MRETVAL=0
             if match_remote_url "$URL"; then
                 # Test for simple HTTP 30X redirection
                 # (disable User-Agent because some proxy can fake it)
@@ -1006,24 +931,17 @@ for ITEM in "${COMMAND_LINE_ARGS[@]}"; do
         fi
 
         if [ $MRETVAL -ne 0 ]; then
-            if match_remote_url "$URL"; then
-                if [ -z "$MODULE" ]; then
-                    log_error "Skip: no module for URL ($(basename_url "$URL"))"
-                else
-                    log_error "Skip: invalid URL (${URL%/}) but module is supported ($MODULE)"
-                fi
-            fi
+            match_remote_url "$URL" && \
+                log_error "Skip: no module for URL ($(basename_url "$URL")/)"
 
             # Check if plowlist can handle $URL
-            if [[ ! $MODULES2 ]]; then
-                declare -a MODULES2=()
-                eval "$(get_all_modules_list list download)" || exit
-                for MODULE in "${!MODULES_PATH[@]}"; do
-                    source "${MODULES_PATH[$MODULE]}"
-                    MODULES2+=("$MODULE")
+            if [[ ! $MODULES_LIST ]]; then
+                mapfile -t MODULES_LIST < <(get_all_modules_list "$LIBDIR" 'list' 'download') || true
+                for MODULE in "${MODULES_LIST[@]}"; do
+                    source "$LIBDIR/modules/$MODULE.sh"
                 done
             fi
-            MODULE=$(get_module "$URL" MODULES2[@]) || true
+            MODULE=$(get_module "$URL" MODULES_LIST[@]) || true
             if [ -n "$MODULE" ]; then
                 log_notice "Note: This URL ($MODULE) is supported by plowlist"
             fi
@@ -1039,13 +957,11 @@ for ITEM in "${COMMAND_LINE_ARGS[@]}"; do
                 "${COMMAND_LINE_MODULE_OPTS[@]}")" || true
 
             [ "${#UNUSED_OPTS[@]}" -eq 0 ] || \
-                log_notice "$MODULE: unused command line switches: ${UNUSED_OPTS[*]}"
+                log_notice "$MODULE: unused command line switches: ${UNUSED_OPTS[@]}"
 
             # Module storage policy (part 1/2)
             if [ "$CACHE" = 'none' ]; then
                 storage_reset
-            elif [ "$CACHE" != 'shared' ]; then
-                [[ ${CACHED_MODULES["$MODULE"]} ]] || CACHED_MODULES["$MODULE"]=1
             fi
 
             ${MODULE}_vars_set
@@ -1069,15 +985,18 @@ done
 
 # Module storage policy (part 2/2)
 if [ "$CACHE" != 'shared' ]; then
-    for MODULE in "${!CACHED_MODULES[@]}"; do storage_reset; done
+    storage_reset
 fi
 
+# Restore umask
+test "$UMASK" && umask $UMASK
+
 if [ ${#RETVALS[@]} -eq 0 ]; then
     exit 0
 elif [ ${#RETVALS[@]} -eq 1 ]; then
     exit ${RETVALS[0]}
 else
-    log_debug "retvals:${RETVALS[*]}"
+    log_debug "retvals:${RETVALS[@]}"
     # Drop success values
     RETVALS=(${RETVALS[@]/#0*} -$ERR_FATAL_MULTIPLE)
 
