@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 #
 # Upload files to file sharing websites
-# Copyright (c) 2010-2016 Plowshare team
+# Copyright (c) 2010-2014 Plowshare team
 #
 # This file is part of Plowshare.
 #
@@ -22,12 +22,11 @@ declare -r VERSION='GIT-snapshot'
 
 declare -r EARLY_OPTIONS="
 HELP,h,help,,Show help info and exit
-HELPFUL,H,longhelp,,Exhaustive help info (with modules command-line options)
+HELPFULL,H,longhelp,,Exhaustive help info (with modules command-line options)
 GETVERSION,,version,,Output plowup version information and exit
 ALLMODULES,,modules,,Output available modules (one per line) and exit. Useful for wrappers.
 EXT_PLOWSHARERC,,plowsharerc,f=FILE,Force using an alternate configuration file (overrides default search path)
-NO_PLOWSHARERC,,no-plowsharerc,,Do not use any plowshare.conf configuration file
-NO_COLOR,,no-color,,Disables log notice & log error output coloring"
+NO_PLOWSHARERC,,no-plowsharerc,,Do not use any plowshare.conf configuration file"
 
 declare -r MAIN_OPTIONS="
 VERBOSE,v,verbose,c|0|1|2|3|4=LEVEL,Verbosity level: 0=none, 1=err, 2=notice (default), 3=dbg, 4=report
@@ -37,9 +36,8 @@ MIN_LIMIT_RATE,,min-rate,r=SPEED,Limit minimum speed to bytes/sec (during 30 sec
 INTERFACE,i,interface,s=IFACE,Force IFACE network interface
 TIMEOUT,t,timeout,n=SECS,Timeout after SECS seconds of waits
 MAXRETRIES,r,max-retries,N=NUM,Set maximum retries for upload failures (fatal, network errors). Default is 0 (no retry).
-NAME_FORMAT,,name,s=FORMAT,Format destination filename (applies on each file argument). Default is \"%f\".
+NAME_FORMAT,,name,s=FORMAT,Format destination filename (applies on each file argument). Default string is: \"%f\".
 CACHE,,cache,C|none|session|shared=METHOD,Policy for storage data. Available: none, session (default), shared.
-TEMP_DIR,,temp-directory,D=DIR,Directory for temporary files (cookies, images)
 CAPTCHA_METHOD,,captchamethod,s=METHOD,Force specific captcha solving method. Available: online, imgur, x11, fb, nox, none.
 CAPTCHA_PROGRAM,,captchaprogram,F=PROGRAM,Call external program/script for captcha solving.
 CAPTCHA_9KWEU,,9kweu,s=KEY,9kw.eu captcha (API) key
@@ -47,9 +45,8 @@ CAPTCHA_ANTIGATE,,antigate,s=KEY,Antigate.com captcha key
 CAPTCHA_BHOOD,,captchabhood,a=USER:PASSWD,CaptchaBrotherhood account
 CAPTCHA_COIN,,captchacoin,s=KEY,captchacoin.com API key
 CAPTCHA_DEATHBY,,deathbycaptcha,a=USER:PASSWD,DeathByCaptcha account
-PRE_COMMAND,,run-before,F=PROGRAM,Call external program/script before new file processing
-PRINTF_FORMAT,,printf,s=FORMAT,Print results in a given format (for each successful upload). Default is \"%L%M%u%n\".
-EXT_CURLRC,,curlrc,f=FILE,Force using an alternate curl configuration file (overrides ~/.curlrc)
+PRINTF_FORMAT,,printf,s=FORMAT,Print results in a given format (for each successful upload). Default string is: \"%D%A%u%n\".
+TEMP_DIR,,temp-directory,D=DIR,Directory for temporary files (cookies, images)
 NO_CURLRC,,no-curlrc,,Do not use curlrc config file"
 
 
@@ -99,7 +96,7 @@ module_exist() {
     local MODULE
 
     for MODULE in "${!1}"; do
-        if [[ $N1 = "$MODULE" || $N2 = "$MODULE" ]]; then
+        if [[ $N1 = $MODULE || $N2 = $MODULE ]]; then
             echo "$MODULE"
             return 0
         fi
@@ -119,17 +116,13 @@ module_config_remote_upload() {
 # Interpreted sequences are:
 # %f: destination (remote) filename
 # %u: download url
-# %U: download url (JSON string)
 # %d: delete url
-# %D: delete url (JSON string)
 # %a: admin url/code
-# %A: admin url/code (JSON string)
 # %l: source (local) filename
 # %m: module name
 # %s: filesize (in bytes)
-# %L: alias for "#DEL %d%n" or empty string (if %d is empty)
-# %M: alias for "#ADM %a%n" or empty string (if %a is empty)
-# %T: time of finished file upload ("HH:MM:SS" = date +%T)
+# %D: alias for "#DEL %d%n" or empty string (if %d is empty)
+# %A: alias for "#ADM %a%n" or empty string (if %a is empty)
 # and also:
 # %n: newline
 # %t: tabulation
@@ -141,7 +134,7 @@ module_config_remote_upload() {
 pretty_check() {
     # This must be non greedy!
     local S TOKEN
-    S=${1//%[fuUdDaAlmsLMntT%]}
+    S=${1//%[fudalmsADnt%]}
     TOKEN=$(parse_quiet . '\(%.\)' <<< "$S")
     if [ -n "$TOKEN" ]; then
         log_error "Bad format string: unknown sequence << $TOKEN >>"
@@ -158,30 +151,28 @@ pretty_print() {
 
     test "${FMT#*%%}" != "$FMT" && FMT=$(replace_all '%%' "%raw" <<< "$FMT")
 
-    if test "${FMT#*%L}" != "$FMT"; then
+    if test "${FMT#*%D}" != "$FMT"; then
         if [ -z "${A[4]}" ]; then
-            FMT=${FMT//%L/}
+            FMT=${FMT//%D/}
         else
-            FMT=$(replace_all '%L' "#DEL %d%n" <<< "$FMT")
+            FMT=$(replace_all '%D' "#DEL %d%n" <<< "$FMT")
         fi
     fi
 
-    if test "${FMT#*%M}" != "$FMT"; then
+    if test "${FMT#*%A}" != "$FMT"; then
         if [ -z "${A[5]}" ]; then
-            FMT=${FMT//%M/}
+            FMT=${FMT//%A/}
         else
-            FMT=$(replace_all '%M' "#ADM %a%n" <<< "$FMT")
+            FMT=$(replace_all '%A' "#ADM %a%n" <<< "$FMT")
         fi
     fi
 
     test "${FMT#*%s}" != "$FMT" && \
-        FMT=$(replace_all '%s' "$(get_filesize "${A[1]}")" <<< "$FMT")
+        FMT=$(replace_all '%s' $(get_filesize "${A[1]}") <<< "$FMT")
 
     handle_tokens "$FMT" '%raw,%' '%t,	' "%n,$CR" \
         "%m,${A[0]}" "%l,${A[1]}" "%f,${A[2]}" "%u,${A[3]}" \
-        "%d,${A[4]}" "%a,${A[5]}" "%U,$(json_escape "${A[3]}")" \
-        "%D,$(json_escape "${A[4]}")" "%A,$(json_escape "${A[5]}")" \
-        "%T,$(date +%T)"
+        "%d,${A[4]}" "%a,${A[5]}"
 }
 
 # Filename (arguments) printf format
@@ -225,10 +216,10 @@ pretty_name_print() {
     test "${FMT#*%%}" != "$FMT" && FMT=$(replace_all '%%' "%raw" <<< "$FMT")
 
     test "${FMT#*%s}" != "$FMT" && \
-        FMT=$(replace_all '%s' "$(get_filesize "${A[1]}")" <<< "$FMT")
+        FMT=$(replace_all '%s' $(get_filesize "${A[1]}") <<< "$FMT")
 
     test "${FMT#*%h}" != "$FMT" && \
-        FMT=$(replace_all '%h' "$(md5_file "${A[1]}")" <<< "$FMT")
+        FMT=$(replace_all '%h' $(md5_file "${A[1]}") <<< "$FMT")
 
     handle_tokens "$FMT" '%raw,%' \
         "%m,${A[0]}" "%l,${A[1]}" "%f,${A[2]}" \
@@ -259,26 +250,16 @@ TMPDIR=${TMPDIR:-/tmp}
 set -e # enable exit checking
 
 source "$LIBDIR/core.sh"
-
-declare -a MODULES=()
-eval "$(get_all_modules_list upload)" || exit
-for MODULE in "${!MODULES_PATH[@]}"; do
-    source "${MODULES_PATH[$MODULE]}"
-    MODULES+=("$MODULE")
+mapfile -t MODULES < <(get_all_modules_list "$LIBDIR" 'upload') || exit
+for MODULE in "${MODULES[@]}"; do
+    source "$LIBDIR/modules/$MODULE.sh"
 done
 
 # Process command-line (plowup early options)
 eval "$(process_core_options 'plowup' "$EARLY_OPTIONS" "$@")" || exit
 
-test "$HELPFUL" && { usage 1; exit 0; }
-if test "$HELP"; then
-    usage
-    if MODULE=$(module_exist MODULES[@] "${UNUSED_OPTS[0]}"); then
-        MODULES=("$MODULE")
-        print_module_options MODULES[@] UPLOAD
-    fi
-    exit 0
-fi
+test "$HELPFULL" && { usage 1; exit 0; }
+test "$HELP" && { usage; exit 0; }
 test "$GETVERSION" && { echo "$VERSION"; exit 0; }
 
 if test "$ALLMODULES"; then
@@ -288,14 +269,7 @@ fi
 
 # Get configuration file options. Command-line is partially parsed.
 test -z "$NO_PLOWSHARERC" && \
-    process_configfile_options '[Pp]lowup' "${MAIN_OPTIONS}
-NO_COLOR,,no-color,,x" "$EXT_PLOWSHARERC"
-
-if [ -n "$NO_COLOR" ]; then
-    unset COLOR
-else
-    declare -r COLOR=yes
-fi
+    process_configfile_options '[Pp]lowup' "$MAIN_OPTIONS" "$EXT_PLOWSHARERC"
 
 declare -a COMMAND_LINE_MODULE_OPTS COMMAND_LINE_ARGS RETVALS
 COMMAND_LINE_ARGS=("${UNUSED_ARGS[@]}")
@@ -311,18 +285,6 @@ elif [ -z "$VERBOSE" ]; then
     declare -r VERBOSE=2
 fi
 
-if [ "${#MODULES}" -le 0 ]; then
-    log_error \
-"-------------------------------------------------------------------------------
-Your plowshare installation has currently no module.
-($PLOWSHARE_CONFDIR/modules.d/ is empty)
-
-In order to use plowup you must install some modules. Here is a quick start:
-$ plowmod --install
--------------------------------------------------------------------------------"
-    exit $ERR_NOMODULE
-fi
-
 if [ $# -lt 1 ]; then
     log_error 'plowup: no module specified!'
     log_error "plowup: try \`plowup --help' for more information."
@@ -374,13 +336,7 @@ else
     [ -n "$CAPTCHA_DEATHBY" ] && log_debug 'plowup: --deathbycaptcha selected'
 fi
 
-if [ -n "$EXT_CURLRC" ]; then
-    if [ -n "$NO_CURLRC" ]; then
-        log_notice 'plowup: --no-curlrc selected and prevails over --curlrc'
-    else
-        log_notice 'plowup: using alternate curl configuration file'
-    fi
-elif [ -z "$NO_CURLRC" -a -f "$HOME/.curlrc" ]; then
+if [ -z "$NO_CURLRC" -a -f "$HOME/.curlrc" ]; then
     log_debug 'using local ~/.curlrc'
 fi
 
@@ -401,11 +357,10 @@ if [ ${#COMMAND_LINE_ARGS[@]} -eq 0 ]; then
 fi
 
 # Check requested module
-if ! MODULE=$(module_exist MODULES[@] "${COMMAND_LINE_ARGS[0]}"); then
-    log_error "plowup: unsupported module (${COMMAND_LINE_ARGS[0]})"
-    log_error "plowup: try \`plowup --modules' to see available modules."
-    exit $ERR_NOMODULE
-fi
+MODULE=$(module_exist MODULES[@] "${COMMAND_LINE_ARGS[0]}") || {
+    log_error "plowup: unsupported module (${COMMAND_LINE_ARGS[0]})";
+    exit $ERR_NOMODULE;
+}
 
 if [ ${#COMMAND_LINE_ARGS[@]} -lt 2 ]; then
     log_error 'plowup: you must specify a filename.'
@@ -421,13 +376,13 @@ eval "$(process_module_options "$MODULE" UPLOAD \
     "${COMMAND_LINE_MODULE_OPTS[@]}")" || true
 
 if [ ${#UNUSED_OPTS[@]} -ne 0 ]; then
-    log_notice "Unused option(s): ${UNUSED_OPTS[*]}"
+    log_notice "Unused option(s): ${UNUSED_OPTS[@]}"
 fi
 
 # Remove module name from argument list
 unset COMMAND_LINE_ARGS[0]
 
-core_init 'plowup'
+set_exit_trap
 
 UCOOKIE=$(create_tempfile) || exit
 URESULT=$(create_tempfile) || exit
@@ -487,42 +442,14 @@ for FILE in "${COMMAND_LINE_ARGS[@]}"; do
         DESTFILE=$(pretty_name_print DATA[@] "$NAME_FORMAT")
     fi
 
+    if match '[;,]' "$DESTFILE"; then
+        log_notice "Skipping ($LOCALFILE): curl can't upload filenames that contain , or ;"
+        continue
+    fi
+
     log_notice "Starting upload ($MODULE): $LOCALFILE"
     log_notice "Destination file: $DESTFILE"
 
-    # Pre-processing script (executed in a subshell)
-    if [ -n "$PRE_COMMAND" ]; then
-        URETVAL=0
-        (exec "$PRE_COMMAND" "$MODULE" "$LOCALFILE" "$DESTFILE" "$UCOOKIE") >/dev/null || URETVAL=$?
-
-        if [ $URETVAL -eq $ERR_NOMODULE ]; then
-            log_notice "Skipping file upload (as requested): $LOCALFILE"
-        elif [ $URETVAL -ne 0 ]; then
-            log_error "Pre-processing script exited with status $URETVAL, continue anyway"
-        fi
-    fi
-
-    SYMLINK=
-    if match '[;,]' "$LOCALFILE"; then
-        SYMLINK="$TMPDIR/$(basename_file "${LOCALFILE//[;,]/_}").$$.$RANDOM"
-        log_notice 'plowup: fix (symlink) local filename that contrain , or ; (because curl cannot upload it)'
-        log_debug "creating: $LOCALFILE => $SYMLINK"
-
-        # Complicated form of $(realpath "$LOCALFILE") but BSD friendly
-        ln -s "$(absolute_path "$LOCALFILE")/$(basename_file "$LOCALFILE")" "$SYMLINK"
-
-        # Swap SYMLINK and LOCALFILE variables (to not lose exact LOCALFILE string)
-        TRY=$LOCALFILE
-        LOCALFILE=$SYMLINK
-        SYMLINK=$TRY
-    fi
-
-    # Workaround if destination filename contain , or ;
-    # Note: Remote sites may not support filenames with such characters.
-    if match '[;,]' "$DESTFILE" && [ '"' != "${DESTFILE:0:1}" -a '"' != "${DESTFILE:(-1):1}" ]; then
-        DESTFILE="\"$DESTFILE\""
-    fi
-
     timeout_init $TIMEOUT
 
     TRY=0
@@ -548,16 +475,6 @@ for FILE in "${COMMAND_LINE_ARGS[@]}"; do
                 log_debug 'arbitrary wait (from module)'
             fi
             wait ${AWAIT:-60} || { URETVAL=$?; break; }
-
-            # Unspecified retry but this error does not count as a retry
-            if [[ $MAXRETRIES -eq 0 ]]; then
-                log_notice "Starting upload ($MODULE): retry (after wait request)"
-                continue
-            fi
-        # Does not count as a retry
-        elif [ $URETVAL -eq $ERR_EXPIRED_SESSION ]; then
-            log_notice "Starting upload ($MODULE): retry (expired session)"
-            continue
         elif [[ $MAXRETRIES -eq 0 ]]; then
             break
         elif [ $URETVAL -ne $ERR_FATAL -a $URETVAL -ne $ERR_NETWORK -a \
@@ -577,11 +494,6 @@ for FILE in "${COMMAND_LINE_ARGS[@]}"; do
 
     ${MODULE}_vars_unset
 
-    if [ -n "$SYMLINK" ]; then
-        rm -f "$LOCALFILE"
-        LOCALFILE=$SYMLINK
-    fi
-
     if [ $URETVAL -eq 0 ]; then
         { read DL_URL; read DEL_URL; read ADMIN_URL_OR_CODE; } <"$URESULT" || true
         if [ -n "$DL_URL" ]; then
@@ -594,7 +506,7 @@ for FILE in "${COMMAND_LINE_ARGS[@]}"; do
 
             DATA=("$MODULE" "$LOCALFILE" "$DESTFILE" \
                   "$DL_URL" "$DEL_URL" "$ADMIN_URL_OR_CODE")
-            pretty_print DATA[@] "${PRINTF_FORMAT:-%L%M%u%n}"
+            pretty_print DATA[@] "${PRINTF_FORMAT:-%D%A%u%n}"
         else
             log_error 'Output URL expected'
             URETVAL=$ERR_FATAL
@@ -631,7 +543,7 @@ if [ ${#RETVALS[@]} -eq 0 ]; then
 elif [ ${#RETVALS[@]} -eq 1 ]; then
     exit ${RETVALS[0]}
 else
-    log_debug "retvals:${RETVALS[*]}"
+    log_debug "retvals:${RETVALS[@]}"
     # Drop success values
     RETVALS=(${RETVALS[@]/#0*} -$ERR_FATAL_MULTIPLE)
 
